//
//  GMlib -- Graphics & Media Lab Common Source Library
//
//  $Id: gmlfilefinder.cpp,v 1.8 2005/04/19 06:21:57 dmoroz Exp $
//
//  Copyright (C) 2004, Moscow State University Graphics & Media Lab
//  gmlsupport@graphics.cs.msu.su
//  
//  This file is part of GMlib software.
//  For conditions of distribution and use, see the accompanying README file.

#include "../base/gmlcommon.h"
#include "../base/gmlstring.h"

#include "gmlfilefinder.h"

#include "gmlpathstring.h"

#include "tchar.h"
#include <windows.h>

using gml::FileFinder;

// forward declarations for static functions

namespace
{
  HANDLE OwnFindFirstDirectory(LPCTSTR lpszSearchFile, LPWIN32_FIND_DATA lpffd);
  BOOL OwnFindNextDirectory(HANDLE hSearch, LPWIN32_FIND_DATA lpffd);
  HANDLE OwnFindFirstFile(LPCTSTR lpszSearchFile, LPWIN32_FIND_DATA lpffd);
  BOOL OwnFindNextFile(HANDLE hSearch, LPWIN32_FIND_DATA lpffd);
}

FileFinder::FileFinder()
{
}

FileFinder::~FileFinder()
{
}

bool FileFinder::FindFiles(const char* filefilter /*= "*"*/,
                           const gml::PathString& /*= ""*/ path,
                           int recursionlevel /*= -1*/)
{
  m_filefilter = filefilter;

#ifndef _WIN32_WCE
  if (path.empty())
    m_startdir.CurDir();
  else
    m_startdir = path;
#else
   ASSERT(!path.empty());
   m_startdir = path;
#endif

  IterateDirToFillInStructure(m_startdir, recursionlevel);

  return true;
}

gml::PathString FileFinder::GetStartDir()
{
  return m_startdir;
}

std::vector<gml::PathString>& FileFinder::GetCompleteFileList()
{
  return m_complete_file_list;
}

std::vector<gml::PathString>& FileFinder::GetSubDirsList()
{
  return m_SubDirs;
}


void FileFinder::IterateDirToFillInStructure(const gml::PathString& cur_path,
                                             int reclev)
{
  DWORD error = 0;
  WIN32_FIND_DATA w32FindBuf;
  LPWIN32_FIND_DATA lpffd = &w32FindBuf;

  if (reclev == 0)
  {
    return;
  }

 #ifndef _WIN32_WCE
  gml::PathString save_cur_path;

  save_cur_path.CurDir();
  bool res = cur_path.ChDir();
  if (res == false) // no such directory
  {
    return;
  }
#endif

  // first find all the files and put them in the List.
#ifndef _WIN32_WCE
  HANDLE hFind = OwnFindFirstFile(m_filefilter.c_str(), lpffd);
#else
  HANDLE hFind = OwnFindFirstFile(
    gml::String::ToUnicode(cur_path + gml::PathString::PATH_SEPARATOR_STR + m_filefilter).c_str(), 
    lpffd);

 
#endif
  if (hFind != INVALID_HANDLE_VALUE)
  {
#ifndef _WIN32_WCE
    gml::PathString file(cur_path, lpffd->cFileName);
#else
    gml::PathString file(cur_path, gml::String::FromUnicode(lpffd->cFileName));
#endif
    m_complete_file_list.push_back(file);
    while (OwnFindNextFile(hFind, lpffd))
    {
#ifndef _WIN32_WCE
      gml::PathString file(cur_path, lpffd->cFileName);
#else
      gml::PathString file(cur_path, gml::String::FromUnicode(lpffd->cFileName));
#endif
       m_complete_file_list.push_back(file);

    }
    FindClose(hFind);
  }

  std::vector<gml::PathString> dirs;
  // investigate the directories
#ifndef _WIN32_WCE
  hFind = OwnFindFirstDirectory(_T("*"), lpffd);
#else
  hFind = OwnFindFirstDirectory(
    gml::String::ToUnicode(cur_path + gml::PathString::PATH_SEPARATOR_STR + "*").c_str(), 
    lpffd);
#endif

  if (hFind != INVALID_HANDLE_VALUE)
  {
#ifndef _WIN32_WCE
    dirs.push_back(lpffd->cFileName);
#else
    dirs.push_back(gml::String::FromUnicode(lpffd->cFileName));
#endif
    while (OwnFindNextDirectory(hFind, lpffd))
    {
#ifndef _WIN32_WCE
    dirs.push_back(lpffd->cFileName);
#else
    dirs.push_back(gml::String::FromUnicode(lpffd->cFileName));
#endif
    }
    FindClose(hFind);
  }

#ifndef _WIN32_WCE
  save_cur_path.ChDir();
#endif
  for (unsigned int teller = 0; teller < dirs.size(); teller++)
  {
    gml::PathString subdir(cur_path, dirs[teller]);
    m_SubDirs.push_back(subdir);
    
    IterateDirToFillInStructure(subdir, reclev - 1);
  }
}
// STATIC FUNCTIONS

namespace
{
  HANDLE OwnFindFirstDirectory(LPCTSTR lpszSearchFile, LPWIN32_FIND_DATA lpffd)
  {
    BOOL bRC;
    //DWORD dwRC;
    HANDLE hSearch;
    int iRC;

    hSearch = ::FindFirstFile(lpszSearchFile, lpffd);
    if (hSearch == INVALID_HANDLE_VALUE)
      return (hSearch);
    for (; ;)
    {
      //dwRC = ::GetFileAttributes(lpffd->cFileName);
      if (lpffd->dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
      {
#ifndef _WIN32_WCE
        iRC = strcmp(lpffd->cFileName, ".");
#else
        iRC = wcscmp(lpffd->cFileName, _T("."));
#endif
        if (iRC)
        {
#ifndef _WIN32_WCE
        iRC = strcmp(lpffd->cFileName, "..");
#else
        iRC = wcscmp(lpffd->cFileName, _T(".."));
#endif
          if (iRC)
            return (hSearch);
        }
      }
      bRC = ::FindNextFile(hSearch, lpffd);
      if (bRC == FALSE)
      {
        FindClose(hSearch);
        return INVALID_HANDLE_VALUE;
      }
    }
  }

  HANDLE OwnFindFirstFile(LPCTSTR lpszSearchFile, LPWIN32_FIND_DATA lpffd)
  {
    BOOL bRC;
    //DWORD dwRC;
    HANDLE hSearch;


    hSearch = ::FindFirstFile(lpszSearchFile, lpffd);


    if (hSearch == INVALID_HANDLE_VALUE)
      return (hSearch);
    for (; ;)
    {
      //dwRC = ::GetFileAttributes(lpffd->cFileName);
      if ((lpffd->dwFileAttributes  & FILE_ATTRIBUTE_DIRECTORY) != FILE_ATTRIBUTE_DIRECTORY) // it is not a directory
      {
        return (hSearch);
      }
      bRC = ::FindNextFile(hSearch, lpffd);
      if (bRC == FALSE)
      {
        FindClose(hSearch);
        return (INVALID_HANDLE_VALUE);
      }
    }
  }

  BOOL OwnFindNextDirectory(HANDLE hSearch, LPWIN32_FIND_DATA lpffd)
  {
    BOOL bRC;
    DWORD dwRC;

    for (; ;)
    {
      bRC = ::FindNextFile(hSearch, lpffd);
      if (bRC == FALSE)
        return (FALSE);
      dwRC = ::GetFileAttributes(lpffd->cFileName);
      if (dwRC & FILE_ATTRIBUTE_DIRECTORY)
        return (TRUE);
    }
  }

  BOOL OwnFindNextFile(HANDLE hSearch, LPWIN32_FIND_DATA lpffd)
  {
    BOOL bRC;
    DWORD dwRC;

    for (; ;)
    {
      bRC = ::FindNextFile(hSearch, lpffd);
      if (bRC == FALSE)
        return (FALSE);
      dwRC = ::GetFileAttributes(lpffd->cFileName);
      if ((dwRC & FILE_ATTRIBUTE_DIRECTORY) != FILE_ATTRIBUTE_DIRECTORY)
      {
        return (TRUE);
      }
    }
  }
}


