// DirTest.cpp : Defines the entry point for the console application.
//

#include <CSL/base/gmlcommon.h>
#include <CSL/files/gmlinifile.h>
//#include "stdafx.h"
#include <calibinit\dmcalibinit.h>
#include <calibinit\dmcalibinternal.h>
#include <CSL/files/gmlfilefinder.h>
#include <highgui.h>

void DrawDetectedPoints(CvPoint2D32f *in_pPoints, int in_iPoints, int in_iObjWdt, CvvImage *out_pImage);
void TestDirectory(const char *in_pcDir);

//=========================================================================================

int main(int argc, char* argv[])
{
  gml::FileFinder theFinder;
  std::vector<gml::PathString> vDirNames;   

  if (argc < 2)
  {
    fprintf(stderr, "Usage: dirtest <folder name>\n");
    return -1;
  };

  const char *pcDir = argv[1];

  //  
  TestDirectory(pcDir);

  //    
  printf("Building folder tree...");
  theFinder.FindFiles("*.*", pcDir, -1);
  printf("done\n");

  //    
  printf("Scanning folders...\n");
  vDirNames = theFinder.GetSubDirsList();

  for (int i = 0; i < vDirNames.size(); i++) 
  {
    if (vDirNames[i].IsDir())
      TestDirectory(vDirNames[i].c_str());
  }

	return 0;
}

//=========================================================================================

void TestDirectory(const char *in_pcDir)
{
  gml::CIniFile   theIniFile;
  gml::FileFinder theFinder;
  std::string     sTemp = in_pcDir;
  std::vector<gml::PathString> vFileNames;     
  int                    iStatus;
  CvPoint2D32f           pvPoints[100];
  int                    iPointCount;

  //   
  gml::PathString sPathTemp (in_pcDir, "tests", "ini");
   
  theIniFile.SetPath(sPathTemp.c_str());
  if (!theIniFile.ReadFile())
  { 
    //fprintf(stderr, " Failed to load ini file with object paramers in '%s'!", in_pcDir);
    return;
  }

  //  
  CvSize                 ObjectSize;

  ObjectSize.width = theIniFile.GetValueI("main", "ObjectWidth") - 1;
  ObjectSize.height = theIniFile.GetValueI("main", "ObjectHeight") - 1;

  if (ObjectSize.width < 2 || ObjectSize.height < 2)
  {  
    return;
  }
  fprintf(stderr, " Valid ini found in '%s'\n", in_pcDir);
  
  //   
  gml::PathString sDir = in_pcDir;
  theFinder.FindFiles("*.jpg", sDir, 1);
  theFinder.FindFiles("*.bmp", sDir, 1);
  theFinder.FindFiles("*.png", sDir, 1);
  
  vFileNames = theFinder.GetCompleteFileList();
  
  //  detection
  for (int i = 0; i < vFileNames.size(); i++) 
  {
    CvvImage SrcImage,
             GrayImage,
             ThreshImage;

    // 
    if (!SrcImage.Load(vFileNames[i].c_str(), 1))
    {
      fprintf(stderr, "  Failed to load test image!\n");
      continue;
    };
    
    GrayImage.CopyOf(SrcImage, 0);
    ThreshImage.CopyOf(SrcImage, 0);

    // 
    try
    {
    
      iStatus = dmFindChessBoardCornerGuesses( GrayImage.GetImage(), ThreshImage.GetImage(),
                                               NULL, ObjectSize, pvPoints, &iPointCount,
                                               1);
    }
    catch (...)
    {
      fprintf(stderr, "  Image %s generated exception\n", i, vFileNames[i].c_str());
      continue;
    }

    // 
    if (iStatus)
    {   
      //fprintf(stderr, "  success\n", vFileNames[i].c_str());

      cvFindCornerSubPix(GrayImage.GetImage(), &pvPoints[0], iPointCount, 
                         cvSize(5,5), cvSize(-1,-1), 
                         cvTermCriteria( CV_TERMCRIT_ITER|CV_TERMCRIT_EPS, 10, 0.01f ));
                       
      DrawDetectedPoints(&pvPoints[0], iPointCount, ObjectSize.width, &SrcImage);

      gml::PathString sOutFile(vFileNames[i].Path() + "\\res", vFileNames[i].Basename(), "jpg");
      sOutFile.MakePath();
      
      SrcImage.Save(sOutFile.c_str());
    }    
    else
    {
      fprintf(stderr, "  Image %s - FAILURE\n", vFileNames[i].c_str());

      // C 
      for (int j = 0; j < DEBUG_IMG_NUM; j++)
      {
        char pcTemp[11];
        sprintf(pcTemp, "_debug%d", j);

        gml::PathString sOutFile(vFileNames[i].Path() + "\\res", vFileNames[i].Basename() + pcTemp, "jpg");
        sOutFile.MakePath();
      
        CvvImage TempImage;
        
        TempImage.CopyOf(g_ppDebugImages[j], 1);
        TempImage.Save(sOutFile.c_str());
      }
    }
  }
}

//=========================================================================================

void DrawDetectedPoints(CvPoint2D32f *in_pPoints, int in_iPoints, int in_iObjWdt, CvvImage *out_pImage)
{
  if( out_pImage->GetImage() && in_iPoints )
  {
      CvMat dst_stub, *dst;
      int count = in_iPoints;
      bool found = true;

      cvScale(out_pImage->GetImage(), out_pImage->GetImage(), 0.5, 64);

      dst = cvGetMat( out_pImage->GetImage(), &dst_stub );

      static const CvScalar line_colors[] =
      {
          {0,0,255},
          {0,128,255},
          {0,200,200},
          {0,255,0},
          {200,200,0},
          {255,0,0},
          {255,0,255}
      };

      const int colorCount = sizeof(line_colors)/sizeof(line_colors[0]);
      const int r = 4;
      CvScalar color = line_colors[0];

      CvPoint prev_pt = { 0, 0};

      for(int j = 0; j < count; j++ )
      {
          CvPoint pt;
          pt.x = cvRound(in_pPoints[j].x);
          pt.y = cvRound(in_pPoints[j].y);

          if( found )
          {
              color = line_colors[(j/cvRound(in_iObjWdt))%colorCount];

              if( j != 0 )
                  cvLine( dst, prev_pt, pt, color, 1, CV_AA );
          }

          cvLine( dst,
                  cvPoint( pt.x - r, pt.y - r ),
                  cvPoint( pt.x + r, pt.y + r ), color, 1, CV_AA );

          cvLine( dst,
                  cvPoint( pt.x - r, pt.y + r),
                  cvPoint( pt.x + r, pt.y - r), color, 1, CV_AA );

          cvCircle( dst, pt, r+1, color, 1, CV_AA );

          prev_pt = pt;
      }
  }
}
